#
# Graph Theoric analysis of ChemNetworks output (graphs)
#
# 6/27/13
#
# Abdullah Ozkanlar 
# abdullah.ozkanlar@wsu.edu
#


library(Matrix)
library(igraph)

#number of frames
numbersteps<-40000

#number of vertices (as printed in .GraphNumnodes)
numbervertices<-216

#modify the path to graphs and the filenames 
all.steps<-paste("/home/abdullah/FROM-ABDULLAH-TO-AURORA-2013/R_script/graphs/water-ethane.input.water",1:numbersteps,".xyz.","ethane",1:numbersteps,".xyz.Graph",sep="")

#Uncomment this if you have different number of nodes/vertices in different frames. Modify the path to graphs and the filenames
#all.nums<-paste("/home/abdullah/FROM-ABDULLAH-TO-AURORA-2013/R_script/graphs/water-ethane.input.water",1:numbersteps,".xyz.","ethane",1:numbersteps,".xyz.GraphNumnodes",sep="")

#the directory where the output files will be written
main.dir<-"/home/abdullah/FROM-ABDULLAH-TO-AURORA-2013/R_script/analysis"

setwd(main.dir)

for(iall in 1:numbersteps){

#Uncomment next three lines if you have different number of nodes/vertices in different frames

#numbervertices<-read.table(all.nums[iall],fill=TRUE)

#numbervertices<-numbervertices[,1:1]

#numbervertices<-as.matrix(numbervertices)

hlist<-read.table(all.steps[iall],fill=TRUE)
    
hlist<-hlist[,1:1]
    
hlist<-as.matrix(hlist)

#construc the graph object
grafik<-graph(edges=hlist,n=(numbervertices),directed=FALSE)    

#simplify the graph by removing any duplicated edges (if any exists)
grafik<-simplify(grafik)
    
grafik.unweighted.net<-grafik

E(grafik.unweighted.net)$weight<-1

#get the degrees of every vertex
no.net.degs<-degree(grafik)

#write the degrees to a file named all.degrees
write(no.net.degs,file="all.degrees",ncolumns=(numbervertices),append=TRUE)

#get the number of vertices in the second order neighborhood of every vertex in the graph
nghbr2<-neighborhood.size(grafik,2,1:(numbervertices))

#write them in the file, neighborhoodsize.order.2
write(nghbr2,file="neighborhoodsize.order.2",ncolumns=(numbervertices),append=TRUE)

nghbr3<-neighborhood.size(grafik,3,1:(numbervertices))

write(nghbr3,file="neighborhoodsize.order.3",ncolumns=(numbervertices+1),append=TRUE)

nghbr4<-neighborhood.size(grafik,4,1:(numbervertices))

write(nghbr4,file="neighborhoodsize.order.4",ncolumns=(numbervertices+1),append=TRUE)

#nghbr5<-neighborhood.size(grafik,5,1:(numbervertices))

#write(nghbr5,file="neighborhoodsize.order.5",ncolumns=(numbervertices+1),append=TRUE)

#nghbr6<-neighborhood.size(grafik,6,1:(numbervertices))

#write(nghbr6,file="neighborhoodsize.order.6",ncolumns=(numbervertices+1),append=TRUE)

#nghbr7<-neighborhood.size(grafik,7,1:(numbervertices))

#write(nghbr7,file="neighborhoodsize.order.7",ncolumns=(numbervertices+1),append=TRUE)

#nghbr8<-neighborhood.size(grafik,8,1:(numbervertices))

#write(nghbr8,file="neighborhoodsize.order.8",ncolumns=(numbervertices+1),append=TRUE)

#nghbr9<-neighborhood.size(grafik,9,1:(numbervertices))

#write(nghbr9,file="neighborhoodsize.order.9",ncolumns=(numbervertices+1),append=TRUE)

}
